/*
 * Lcd.c
 *
 * Created: 13.12.2011 19:10:00
 *  Author: Roland
 */ 


#include "Lcd.h"
#include <avr/io.h>
#include "Timer.h"

static uint8_t Lcd_PosX;
static uint8_t Lcd_PosY;



// AtMega HW Definition for LCD Interface (4Bit)

#define LCD_BACK_LIGHT_MASK (0x08)
#define LCD_BACK_LIGHT_PORT PORTB
#define LCD_BACK_LIGHT_PORT_DDR DDRB

#define LCD_RS_MASK 0x01
#define LCD_RS_PORT PORTA
#define LCD_RS_PORT_DDR DDRA

#define LCD_RW_MASK 0x02
#define LCD_RW_PORT PORTA
#define LCD_RW_PORT_DDR DDRA

#define LCD_E_MASK 0x04
#define LCD_E_PORT PORTA
#define LCD_E_PORT_DDR DDRA

#define LCD_DATA_MASK 0xf0
#define LCD_DATA_PORT PORTA
#define LCD_DATA_PORT_DDR DDRA


void Lcd_SetBackLight(uint8_t x)
{
	OCR0=x;

	//if(onoff==0)
	//{
	//	LCD_BACK_LIGHT_PORT&=~LCD_BACK_LIGHT_MASK;
	//}
	//else
	//{
	//	LCD_BACK_LIGHT_PORT|=LCD_BACK_LIGHT_MASK;
	//}
	
	return;
}

static void Lcd_Cmd4Bit(uint8_t x)
{
	// RS:0
	LCD_RS_PORT&=~LCD_RS_MASK; // Display Command

	// Lo Nibble x auf DB4-7
	LCD_DATA_PORT&=~LCD_DATA_MASK;
	LCD_DATA_PORT|=(x&0x0f)<<4; // Hier fix oberen 4 Bit
	LCD_E_PORT&=~LCD_E_MASK; // /E=L
	LCD_E_PORT|=LCD_E_MASK; // /E=H
	
	return;
}



static void Lcd_Cmd(uint8_t x)
{
	// RS:0
	LCD_RS_PORT&=~LCD_RS_MASK; // Display Command

	// Hi Nibble
	LCD_DATA_PORT&=~LCD_DATA_MASK;
	LCD_DATA_PORT|=(x&0xf0); // Hier fix oberen 4 Bit
	LCD_E_PORT&=~LCD_E_MASK; // /E=L
	LCD_E_PORT|=LCD_E_MASK; // /E=H
	
	Timer_Delayus(100u+10u);
	
	// Lo Nibble
	LCD_DATA_PORT&=~LCD_DATA_MASK;
	LCD_DATA_PORT|=(x&0x0f)<<4; // Hier fix oberen 4 Bit
	LCD_E_PORT&=~LCD_E_MASK; // /E=L
	LCD_E_PORT|=LCD_E_MASK; // /E=H
	
	Timer_Delayus(100u+10u);
	
	return;
	}

static void Lcd_Data(uint8_t x)
{
	// RS: 1
	LCD_RS_PORT|=LCD_RS_MASK; // Display Command

	// Hi Nibble
	LCD_DATA_PORT&=~LCD_DATA_MASK;
	LCD_DATA_PORT|=(x&0xf0); // Hier fix oberen 4 Bit
	LCD_E_PORT&=~LCD_E_MASK; // /E=L
	LCD_E_PORT|=LCD_E_MASK; // /E=H

	Timer_Delayus(100u+10u);

	// Lo Nibble
	LCD_DATA_PORT&=~LCD_DATA_MASK;
	LCD_DATA_PORT|=(x&0x0f)<<4; // Hier fix oberen 4 Bit
	LCD_E_PORT&=~LCD_E_MASK; // /E=L
	LCD_E_PORT|=LCD_E_MASK; // /E=H
	Timer_Delayus(100u+10u);

	return;
	}


void Lcd_Init()
{
	LCD_BACK_LIGHT_PORT_DDR|=LCD_BACK_LIGHT_MASK; 

	LCD_E_PORT|=LCD_E_MASK; // Inaktiv
	//LCD_RS_PORT&=~LCD_RS_MASK;
	LCD_RW_PORT&=~LCD_RW_MASK; // Default: Write (0)
			

	LCD_E_PORT_DDR|=LCD_E_MASK; 
	LCD_RS_PORT_DDR|=LCD_RS_MASK; 
	LCD_RW_PORT_DDR|=LCD_RW_MASK; 

	LCD_DATA_PORT_DDR|=LCD_DATA_MASK; // Daten auf Ausgang
	
	// Initialize Timer 0
	
	TCCR0=0x69; //Fast PWM CLKDIV=1 OC0 Out non inverted
	OCR0=0x00;
		
	Lcd_SetBackLight(0x00); //Backlight off

	Timer_Delayus(15000u+100u);
	Lcd_Cmd4Bit(0x3);  
	Timer_Delayus(4100u+100u);
	Lcd_Cmd4Bit(0x3); 
	Timer_Delayus(100u+10u);
	Lcd_Cmd4Bit(0x3); 
	Lcd_Cmd4Bit(0x2); 

	Timer_Delayus(110u);
	Lcd_Cmd(0x28); // Set Funktion 4 bit data lengt 2 Zeilen
	Timer_Delayus(110u);
	Lcd_Cmd(0x08); // Display Off, Cursor Off, Blinking Off
	Timer_Delayus(110u);
	Lcd_Cmd(0x01); // Display Clear
	Timer_Delayus(1650u);
	Lcd_Cmd(0x06); // Entry Mode Set: C Move increase, not shiftet
	Timer_Delayus(110u);

	Lcd_Cmd(0x10); // Shift: C Move
	Timer_Delayus(110u);

	Lcd_Cmd(0x0c); // Display On, Cursor Off, Blinking Off
	Timer_Delayus(110u);

	Lcd_PosX=0;
	Lcd_PosY=0;
	return;
}

void Lcd_SetPos(int8_t x, int8_t y)
{
	uint8_t adr;
	if(x>15) x=15;
	if(y>3) y=3;
	switch (y)
		{
		case 0:
			adr=0x00;
			break;
			
		case 1:
			adr=0x40;
			break;
			
		case 2:
			adr=0x10;
			break;

		case 3:
			adr=0x50;
			break;
			
		default:
			adr=0x00;
			break;

		}
	adr+=x;
	Lcd_Cmd(0x80+adr); // Set DDRAM Adress
	Timer_Delayus(50u);
	Lcd_PosX=x;
	Lcd_PosY=y;
	return;
}

void Lcd_Clr()
{
	Lcd_Cmd(0x01); // Delete Display, Cursor Home
	Timer_Delayus(1650u);
	Lcd_PosX=0;
	Lcd_PosY=0;
	return;
}

void Lcd_PutC(char c)
{
	switch(c)
	{
		case '\n':
			Lcd_PosX=0;
			Lcd_PosY++;
			if(Lcd_PosY>3) Lcd_PosY=3;
			Lcd_SetPos(Lcd_PosX,Lcd_PosY);
			break;
		
		default:
			Lcd_Data(c);
			Timer_Delayus(45u);
			Lcd_PosX++;
			if(Lcd_PosX>15)
			{
				Lcd_PosX=0;
				Lcd_PosY++;  
				if(Lcd_PosY>3) Lcd_PosY=3;
				Lcd_SetPos(Lcd_PosX,Lcd_PosY);
			}				
			break;

	}

	return;
}

void Lcd_PutS(char *s)
{
	while(*s)
	{
		Lcd_PutC(*s++);
	}
	return;
}
